# Stub-only types. This module does not exist at runtime.

from typing import Any, Optional
from typing_extensions import Protocol

# As defined https://docs.python.org/3/library/xml.dom.html#domimplementation-objects
class DOMImplementation(Protocol):
    def hasFeature(self, feature: str, version: Optional[str]) -> bool: ...
    def createDocument(self, namespaceUri: str, qualifiedName: str, doctype: Optional[Any]) -> Any: ...
    def createDocumentType(self, qualifiedName: str, publicId: str, systemId: str) -> Any: ...



from abc import abstractmethod
from distutils.dist import Distribution
from typing import Any, Callable, Iterable, List, Optional, Text, Tuple, Union

class Command:
    sub_commands: List[Tuple[str, Optional[Callable[[Command], bool]]]]
    def __init__(self, dist: Distribution) -> None: ...
    @abstractmethod
    def initialize_options(self) -> None: ...
    @abstractmethod
    def finalize_options(self) -> None: ...
    @abstractmethod
    def run(self) -> None: ...
    def announce(self, msg: Text, level: int = ...) -> None: ...
    def debug_print(self, msg: Text) -> None: ...
    def ensure_string(self, option: str, default: Optional[str] = ...) -> None: ...
    def ensure_string_list(self, option: Union[str, List[str]]) -> None: ...
    def ensure_filename(self, option: str) -> None: ...
    def ensure_dirname(self, option: str) -> None: ...
    def get_command_name(self) -> str: ...
    def set_undefined_options(self, src_cmd: Text, *option_pairs: Tuple[str, str]) -> None: ...
    def get_finalized_command(self, command: Text, create: int = ...) -> Command: ...
    def reinitialize_command(self, command: Union[Command, Text], reinit_subcommands: int = ...) -> Command: ...
    def run_command(self, command: Text) -> None: ...
    def get_sub_commands(self) -> List[str]: ...
    def warn(self, msg: Text) -> None: ...
    def execute(self, func: Callable[..., Any], args: Iterable[Any], msg: Optional[Text] = ..., level: int = ...) -> None: ...
    def mkpath(self, name: str, mode: int = ...) -> None: ...
    def copy_file(
        self,
        infile: str,
        outfile: str,
        preserve_mode: int = ...,
        preserve_times: int = ...,
        link: Optional[str] = ...,
        level: Any = ...,
    ) -> Tuple[str, bool]: ...  # level is not used
    def copy_tree(
        self,
        infile: str,
        outfile: str,
        preserve_mode: int = ...,
        preserve_times: int = ...,
        preserve_symlinks: int = ...,
        level: Any = ...,
    ) -> List[str]: ...  # level is not used
    def move_file(self, src: str, dst: str, level: Any = ...) -> str: ...  # level is not used
    def spawn(self, cmd: Iterable[str], search_path: int = ..., level: Any = ...) -> None: ...  # level is not used
    def make_archive(
        self,
        base_name: str,
        format: str,
        root_dir: Optional[str] = ...,
        base_dir: Optional[str] = ...,
        owner: Optional[str] = ...,
        group: Optional[str] = ...,
    ) -> str: ...
    def make_file(
        self,
        infiles: Union[str, List[str], Tuple[str]],
        outfile: str,
        func: Callable[..., Any],
        args: List[Any],
        exec_msg: Optional[str] = ...,
        skip_msg: Optional[str] = ...,
        level: Any = ...,
    ) -> None: ...  # level is not used



import collections
from typing import Any, Dict, Generic, Iterable, Iterator, Mapping, Tuple, Type, TypeVar, overload

_S = TypeVar("_S")
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")

class frozendict(Mapping[_KT, _VT], Generic[_KT, _VT]):

    dict_cls: Type[Dict] = ...
    @overload
    def __init__(self, **kwargs: _VT) -> None: ...
    @overload
    def __init__(self, mapping: Mapping[_KT, _VT]) -> None: ...
    @overload
    def __init__(self, iterable: Iterable[Tuple[_KT, _VT]]) -> None: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    def __contains__(self, key: object) -> bool: ...
    def copy(self: _S, **add_or_replace: _VT) -> _S: ...
    def __iter__(self) -> Iterator[_KT]: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __hash__(self) -> int: ...

class FrozenOrderedDict(frozendict):

    dict_cls: Type[collections.OrderedDict] = ...
